package com.isu.gaswellwatch.modbus.data.impl;

import com.isu.gaswellwatch.modbus.data.PersistenceHandler;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.PreparedStatementCallback;
import org.springframework.stereotype.Component;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/24 17:33
 */
@Component(PersistenceHandler.KNPCV1_MODBUS_TYPE)
public class Knpcv1PersistenceHandler extends AbstractPersistenceHandler {

    private static final Map<String, String> KNPC_V1_COLUMN_MAPPING_MAP = new HashMap<>();

    static {
        KNPC_V1_COLUMN_MAPPING_MAP.put("well_status", "wellStatus");
        KNPC_V1_COLUMN_MAPPING_MAP.put("plug_status", "plugStatus");
        KNPC_V1_COLUMN_MAPPING_MAP.put("status_start_time", "statusOpenTime");
        KNPC_V1_COLUMN_MAPPING_MAP.put("status_end_time", "statusCloseTime");
        KNPC_V1_COLUMN_MAPPING_MAP.put("oil_pressure", "oilPressure");
        KNPC_V1_COLUMN_MAPPING_MAP.put("cas_pressure", "casPressure");
        KNPC_V1_COLUMN_MAPPING_MAP.put("pre_pressure", "prePressure");
        KNPC_V1_COLUMN_MAPPING_MAP.put("pipe_pressure", "pipePressure");
        KNPC_V1_COLUMN_MAPPING_MAP.put("liquid_level", "liquidLevel");
        KNPC_V1_COLUMN_MAPPING_MAP.put("temperature", "temperature");
        KNPC_V1_COLUMN_MAPPING_MAP.put("humidity", "humidity");
        KNPC_V1_COLUMN_MAPPING_MAP.put("op_mode", "opmode");
        KNPC_V1_COLUMN_MAPPING_MAP.put("timer_able1", "timer1");
        KNPC_V1_COLUMN_MAPPING_MAP.put("timer_able2", "timer2");
        KNPC_V1_COLUMN_MAPPING_MAP.put("timer_open1", "timer1Open");
        KNPC_V1_COLUMN_MAPPING_MAP.put("timer_close1", "timer1Close");
        KNPC_V1_COLUMN_MAPPING_MAP.put("timer_open2", "timer2Open");
        KNPC_V1_COLUMN_MAPPING_MAP.put("timer_close2", "timer2Close");
        KNPC_V1_COLUMN_MAPPING_MAP.put("timing_open", "timingOpen");
        KNPC_V1_COLUMN_MAPPING_MAP.put("timing_close", "timingClose");
        KNPC_V1_COLUMN_MAPPING_MAP.put("timing_delay", "timingCelay");
        KNPC_V1_COLUMN_MAPPING_MAP.put("presource", "presource");
        KNPC_V1_COLUMN_MAPPING_MAP.put("pressure_open", "pressureOpen");
        KNPC_V1_COLUMN_MAPPING_MAP.put("pressure_close", "pressureClose");
        KNPC_V1_COLUMN_MAPPING_MAP.put("trigger_type", "triggerType");
        KNPC_V1_COLUMN_MAPPING_MAP.put("stability_time", "stabilityTime");
        KNPC_V1_COLUMN_MAPPING_MAP.put("max_open_well", "maxOpenWell");
        KNPC_V1_COLUMN_MAPPING_MAP.put("max_close_well", "maxCloseWell");
        KNPC_V1_COLUMN_MAPPING_MAP.put("min_open_well", "minOpenWell");
        KNPC_V1_COLUMN_MAPPING_MAP.put("min_close_well", "minCloseWell");
        KNPC_V1_COLUMN_MAPPING_MAP.put("prespro_tect", "presproTect");
        KNPC_V1_COLUMN_MAPPING_MAP.put("prespro_source", "presproSource");
        KNPC_V1_COLUMN_MAPPING_MAP.put("open_well_limit_max", "openWellLimitMax");
        KNPC_V1_COLUMN_MAPPING_MAP.put("open_well_limit_min", "openWellLimitMin");
        KNPC_V1_COLUMN_MAPPING_MAP.put("plug_init_status", "plugInitStatus");
        KNPC_V1_COLUMN_MAPPING_MAP.put("plug_sustain_time", "plugSustainTime");
        KNPC_V1_COLUMN_MAPPING_MAP.put("plug_close_time", "plugCloseTime");
        KNPC_V1_COLUMN_MAPPING_MAP.put("tp_init_status", "tpInitStatus");
        KNPC_V1_COLUMN_MAPPING_MAP.put("tp_open_source", "tpOpenSource");
        KNPC_V1_COLUMN_MAPPING_MAP.put("tp_open_trigger", "tpOpenTrigger");
        KNPC_V1_COLUMN_MAPPING_MAP.put("tp_open_pressure", "tpOpenPressure");
        KNPC_V1_COLUMN_MAPPING_MAP.put("tp_open_time", "tpOpenTime");
        KNPC_V1_COLUMN_MAPPING_MAP.put("tp_close_source", "tpCloseSource");
        KNPC_V1_COLUMN_MAPPING_MAP.put("tp_close_trigger", "tpCloseTrigger");
        KNPC_V1_COLUMN_MAPPING_MAP.put("tp_close_pressure", "tpClosePressure");
        KNPC_V1_COLUMN_MAPPING_MAP.put("tp_close_time", "tpCloseTime");
        KNPC_V1_COLUMN_MAPPING_MAP.put("tp_stability_time", "tpStabilityTime");
    }

    @Override
    public void createTable(String tableName, Long deviceId) {
        this.createTable("sql/CREATE_KNPCV1.sql", tableName, deviceId);
    }

    @Override
    public Map<String, Map<String, Object>> insert(String tableName, String cacheKey) {
        String insertTableSQL = this.getResource("sql/INSERT_KNPCV1.sql");
        insertTableSQL = StringUtils.replace(insertTableSQL, "$TableName$", tableName);

        Map<String, Object> oldRow = this.getLastRow(tableName);
        Map<String, Object> newRow = this.redisTemplate.opsForHash().entries(cacheKey);

        this.jdbcTemplate.execute(insertTableSQL, new PreparedStatementCallback<Object>() {
            @Override
            public Object doInPreparedStatement(PreparedStatement ps) throws SQLException {
                ps.setLong(1, Knpcv1PersistenceHandler.this.snowflakeConfig.snowflakeId());
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 2, "deviceId", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 3, "collectionTime", Types.TIMESTAMP);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 4, "receiveTime", Types.TIMESTAMP);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 5, "deviceTime", Types.TIMESTAMP);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 6, "runMode", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 7, "wellStatus", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 8, "plugStatus", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 9, "statusOpenTime", Types.VARCHAR);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 10, "statusCloseTime", Types.VARCHAR);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 11, "oilPressure", Types.DECIMAL);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 12, "casPressure", Types.DECIMAL);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 13, "prePressure", Types.DECIMAL);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 14, "pipePressure", Types.DECIMAL);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 15, "liquidLevel", Types.DECIMAL);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 16, "temperature", Types.DECIMAL);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 17, "humidity", Types.DECIMAL);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 18, "opmode", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 19, "timer1", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 20, "timer2", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 21, "timer1Open", Types.VARCHAR);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 22, "timer1Close", Types.VARCHAR);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 23, "timer2Open", Types.VARCHAR);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 24, "timer2Close", Types.VARCHAR);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 25, "timingOpen", Types.VARCHAR);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 26, "timingClose", Types.VARCHAR);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 27, "timingCelay", Types.VARCHAR);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 28, "presource", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 29, "pressureOpen", Types.DECIMAL);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 30, "pressureClose", Types.DECIMAL);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 31, "triggerType", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 32, "stabilityTime", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 33, "maxOpenWell", Types.VARCHAR);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 34, "maxCloseWell", Types.VARCHAR);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 35, "minOpenWell", Types.VARCHAR);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 36, "minCloseWell", Types.VARCHAR);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 37, "presproTect", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 38, "presproSource", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 39, "openWellLimitMax", Types.DECIMAL);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 40, "openWellLimitMin", Types.DECIMAL);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 41, "plugInitStatus", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 42, "plugSustainTime", Types.VARCHAR);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 43, "plugCloseTime", Types.VARCHAR);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 44, "tpInitStatus", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 45, "tpOpenSource", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 46, "tpOpenTrigger", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 47, "tpOpenPressure", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 48, "tpOpenTime", Types.VARCHAR);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 49, "tpCloseSource", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 50, "tpCloseTrigger", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 51, "tpClosePressure", Types.INTEGER);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 52, "tpCloseTime", Types.VARCHAR);
                Knpcv1PersistenceHandler.this.setValue(ps, newRow, 53, "tpStabilityTime", Types.INTEGER);
                return ps.executeUpdate();
            }
        });
        if (Objects.isNull(oldRow)) {
            return Map.of("new", newRow);
        }
        return Map.of("new", newRow, "old", oldRow);
    }

    @Override
    protected Map<String, String> getColumnMappingMap() {
        return KNPC_V1_COLUMN_MAPPING_MAP;
    }
    
}
